/*
 *     ds18b20
 *       ,     2  3 .
 *      .  3    ,   
 * ver 1.3
 */ 

#include "bits_macros.h"
#include "OneWare.h"
#include "ds18b20.h"
#include "sensors.h"
#include "EERTOS.h"

//-------------  DS18B20
#define DS18B20_START			0x44			//  
#define DS18B20_READ			0xbe			// 
#define DS18B20_REPEAT_READ		500				//       1-ware 
#define DS18B20_ERRPEPEAT_READ	200				//     

#define DS_REPEAT_0XFF			3				//     0xff,   

void StartMeasureDS18(void);					//  
void ErrorEvent(void);							//    

/************************************************************************/
/*   ,    			*/
/************************************************************************/
void ReadMeasurementComplet(void){
	u08 mOk = 0,
		val = ((owRecivBuf[0]>>4) & 0xf) | ((owRecivBuf[1]<<4) & 0xf0);	//    .  ds18b20    4      4   
	static u08 i=0, m1, m2, PreFF = 0;

	switch (i){
		case 0:									//   
			m1 = val;
			break;
		case 1:
			if (m1 == val){						//     ,    
				mOk = 1;
			}
			m2 = val;							//     
			break;
		case 3:
			if ((m2 == val) || (m1 == val)){		//     ,  
				mOk = 1;
			}
			i = 0; m1=0; m2=0;					//    .   
			break;
		default:
			break;
	}
	if (mOk){									//   
		i = 0; m1 = 0; m2 = 0;
		if ((val != 0xff) || (PreFF > DS_REPEAT_0XFF)){		//          3 
			PreFF = 0;
			struct sSensor Sens;				//  ,    ,      
			Sens.State = 0;
			SensorSetInBus(Sens);				//   
			SetSensor(SENSOR_DS18B20, Sens.State, val);	//   
			if ((ClockStatus == csSensorSet) && (SetStatus == ssSensWaite)){	//   ,    
				Refresh();
				SetTimerTask(StartMeasureDS18, DS18B20_ERRPEPEAT_READ);	//      
			}
			return;
		}
		PreFF++;
	}
	i++;										// 
	SetTimerTask(StartMeasureDS18, DS18B20_ERRPEPEAT_READ);		//    
}

/************************************************************************/
/*   							            */
/************************************************************************/
void StartReadMeasurement(void){
	if owIsFree(){								// ?
		owSendBuf[0] = ONE_WARE_SKIP_ROM;
		owSendBuf[1] = DS18B20_READ;
		owSendAndReciv();						//  
		owCompleted = ReadMeasurementComplet;
		owExchage();
	}
	else{
		SetTimerTask(StartReadMeasurement, DS18B20_REPEAT_READ);
	}
}

/************************************************************************/
/*     ,                */
/************************************************************************/
void WaitEndMeasure(void){
	SetTimerTask(StartReadMeasurement, DS18B20_REPEAT_READ);	//  
}

/************************************************************************/
/*                                                    */
/************************************************************************/
void StartMeasureDS18(void){
	if owIsFree(){								// ?
		owSendBuf[0] = ONE_WARE_SKIP_ROM;
		owSendBuf[1] = DS18B20_START;
		owSendOnly();							// 
		owCompleted = WaitEndMeasure;
		owError = ErrorEvent;
		owExchage();							//    1-Ware
	}
	else{										// ,   
		SetTimerTask(StartMeasureDS18, DS18B20_REPEAT_READ);
	}
}

/************************************************************************/
/*                                                  */
/************************************************************************/
void ErrorEvent(void){
	struct sSensor Sens;						//  ,    ,      
	Sens.State = 0;
	SensorNoInBus(Sens);						//    
	SetSensor(SENSOR_DS18B20, Sens.State, 0);			//  
	SetTimerTask(StartMeasureDS18, DS18B20_REPEAT_READ);					//      
	if ((ClockStatus == csSensorSet) && (SetStatus == ssSensWaite))	//   ,    
		Refresh();
}
